/***************************************************************************
               ulxr_http_prootocol.h  -  http prootocol
                             -------------------
    begin                : Mon May 3 2004
    copyright            : (C) 2002-2004 by Ewald Arnold
    email                : ulxmlrpcpp@ewald-arnold.de

    $Id: ulxr_http_protocol.h,v 1.5 2004/05/19 15:53:13 ewald-arnold Exp $

 ***************************************************************************/

/**************************************************************************
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ***************************************************************************/

#ifndef ULXR_HTTP_PROTOCOL_H
#define ULXR_HTTP_PROTOCOL_H

#include <ulxmlrpcpp.h>  // always first header

#include <map>
#include <vector>

#include <ulxr_protocol.h>


namespace ulxr {


/** Runs http as protocol for rpc transmition.
  */
class ULXR_EXPORT HttpProtocol : public Protocol
{
 public:
   typedef std::map<CppString, CppString> header_property;

 /** Constructs a Protocol.
   * @param  conn      pointer to connection object
   * @param  hostname  name of host
   */
   HttpProtocol(Connection *conn, const CppString &hostname);

 /** Destroys the Protocol.
   */
   virtual ~HttpProtocol();

 /** Tests if the response was successful regarding the transportation.
   * @param   phrase  return value describing the problem.
   * @return true  resonse is OK.
   */
   virtual bool responseStatus(CppString &phrase) const;

 /** Sends a MethodCall over the connection.
   * @param   call      pointer to the calling data
   * @param   resource  resource for rpc on remote host
   * @param   wbxml     true: data is sent as wbxml
   */
   virtual void sendRpcCall(const MethodCall &call, const CppString &resource, bool wbxml);

 /** Sends a MethodResponse over the connection.
   * @param   call   pointer to the response data
   * @param   wbxml  true: data is sent as wbxml
   */
   virtual void sendRpcResponse(const MethodResponse &resp, bool wbxml);

 /** Resets the state of the Protocol.
   * Before starting a transfer you should call this to ensure
   * a defined state of the internal state machine processing the
   * protocol.
   */
   virtual void resetConnection();

 /** Process a chunk of input data from the connection.
   * Usually you call read() to get some more data and pass it to
   * this method. It processes the header.
   * Normally you only have to take care of two states: ConnError which
   * results in terminating the connection. ConnBody on the other side
   * indicates user data which might be fed to a parser. @see Dispatcher.
   * @param  buffer   pointer to input data
   * @param len       valid length of buffer
   * @return   current state of the processing.
   */
   virtual State connectionMachine(char * &buffer, long &len);

 /** Sets the current transmission to expect/accept no return value.
   */
   virtual void setTransmitOnly();

 /** Tests if the current transmission expects a return value.
   * @return true: return value for request
   */
   virtual bool isTransmitOnly();

  /** Returns the protocol name.
   * @return protocol name
   */
   virtual CppString getProtocolName();

 /** Gets the number of bytes to read from the message body.
   * @return amount of bytes to read, -1 means unknown length.
   */
   virtual long getBytesToRead() const;

 /** Determines what to do after completing the current transaction.
   * Depending on the protocol version and some properties the connection
   * remains open or closes.
   * @param  http_ver   numeral part of the version from the http header
   * @return true: close connection afterwards
   */
   virtual bool determineClosing(const CppString &http_ver);

 //////////////////////////////////////////////////////////////////////////////////
 /// http stuff

 /** Sends a header and a html body containing error values from "outside".
   * @param  status  http error code
   * @param  phrase  human readable description
   * @param  info    additionaly info to appear in the html response part
   */
   void sendNegativeResponse(int status, const CppString &phrase,
                             const CppString &info = ULXR_PCHAR(""));

 /** Sends a http request header.
   * The actual data must follow directly afterwards.
   * @param  method     the http method in use
   * @param  resource   the requested resource
   * @param  type       the content-type of the requesting data
   * @param  len        the length of the data in the requesting body
   * @param  wbxml_mode true: suppress some header fields
   */
   void sendRequestHeader(const CppString &method,
                          const CppString &resource,
                          const CppString &type, unsigned len,
                          bool wbxml_mode = false);

 /** Sends a http response header.
   * The actual data must follow directly afterwards.
   * @param  code       http status code
   * @param  resource   human readable http status phrase
   * @param  type       the content-type of the requesting data
   * @param  len        the length of the data in the requesting body
   * @param  wbxml_mode true: suppress some header fields
   */
   void sendResponseHeader(int code,
                           const CppString &phrase,
                           const CppString &type,
                           unsigned len,
                           bool wbxml_mode = false);

 /** Gets the user agent.
   * The user agent describes the client application.
   * @return  the user agent.
   */
   CppString getUserAgent() const
     { return useragent; }

 /** Sets the user agent.
   * The user agent describes the client application.
   * @param  the user agent.
   */
   void setUserAgent(const CppString &ua)
     {useragent = ua; }

 /** Sends a negative response t the caller
   * @param  realm   Description for the caller which username and password to use
   * @return true: access shall be granted.
   */
   virtual void rejectAuthentication(const CppString &realm);

 /** Sets username and password for the next transmission (client mode).
   * This makes the next transmission block use a simple authentication scheme
   * with a username and password.
   * @param  user   Username
   * @param  pass   Password
   */
   virtual void setMessageAuthentication(const CppString &user, const CppString &pass);

 /** Gets a property field from the http header.
   * The names are case insensitive. If the property is not available
   * RuntimeException is throw so you should before with hasHttpProperty.
   * @param   name   the name of the field
   * @return  the content of the field.
   */
   CppString getHttpProperty (const CppString &name) const;

 /** Tests if a property field from the http header is available.
   * The names are case insensitive.
   * @param   name   the name of the field
   * @return  true if the property field is available
   */
   bool hasHttpProperty (const CppString &name) const;

 /** Gets the current date as valid string for a http header.
   * @return  date as string
   */
   CppString getDateStr() const;

 /** Gets the first line from the http header.
   * the forst line contains special information depending on the
   * direction from or to the server.
   * @return  date as string
   */
   CppString getFirstHeaderLine() const
     { return header_firstline; }

 /** Detaches the protocol by creating a duplicate of
   * the protocol + connection and closing the original connection afterwards.
   * @return the current protocol
   */
   virtual Protocol *detach();

protected:

 /** Extracts username and password from current message.
   * @param  user   reference to return username
   * @param  user   reference to return password
   * @return true: username and password could be extracted
   */
   virtual bool getUserPass(CppString &user, CppString &pass) const;

 /** Adds a http header field for the next transmission.
   * The user fields are reset after transmission.
   * @param name     http header field name
   * @param value    value of header field
   */
   void addOneTimeHttpField(const CppString &name, const CppString &value);

 /** Determines the length of the message body.
   */
   virtual void determineContentLength();

 /** Parses the first line of the header.
   * The first line contains special information like path to the
   * data, http version in use or error information.
   */
   void parseHeaderLine();

 private:

   CppString    conn_username;
   CppString    conn_password;
   CppString    useragent;
   CppString    header_firstline;
   CppString    header_buffer;
   CppString    hostname;

   std::vector<CppString>   userTempFields;
   header_property          headerprops;

   HttpProtocol & operator= (const HttpProtocol &);

 /** Initializes internal variables.
   */
   void init ();
};


};  // namespace ulxr


#endif // ULXR_HTTP_PROTOCOL_H
