/***************************************************************************
                    ulxr_protocol.h  -  rpc prootocol
                             -------------------
    begin                : Mon May 3 2004
    copyright            : (C) 2002-2004 by Ewald Arnold
    email                : ulxmlrpcpp@ewald-arnold.de

    $Id: ulxr_protocol.h,v 1.6 2004/06/03 16:57:06 ewald-arnold Exp $

 ***************************************************************************/

/**************************************************************************
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ***************************************************************************/

#ifndef ULXR_PROTOCOL_H
#define ULXR_PROTOCOL_H

#include <ulxmlrpcpp.h>  // always first header

#include <vector>


namespace ulxr {


class MethodCall;
class MethodResponse;
class Connection;

/** Base class for the protocol of the rpc transportation.
  */
class ULXR_EXPORT Protocol
{
 public:

 /** Constructs a Protocol.
   * @param  conn  pointer to connection object
   */
   Protocol(Connection *conn);

 /** Destroys the Protocol.
   */
   virtual ~Protocol();

 /** Writes data to the connection.
   * @param  buff pointer to data
   * @param  len  valid buffer length
   */
   void writeRaw(char const *buff, long len);

 /** Reads data from the connection.
   * @param  buff pointer to data buffer
   * @param  len  maimum number of bytes to read into buffer
   * @return number of actually read bytes
   */
   long readRaw(char *buff, long len);

 /** Opens the connection in rpc client mode.
   */
   void open();

 /** Opens the connection in rpc server mode, thus waiting for
   * connections from clients.
   */
   void accept() ;

 /** Tests if the connection is open.
   * @return true if connection is already open.
   */
   bool isOpen() const;

 /** Closes the connection.
   */
   void close();

 /** Gets the number of bytes to read from the message body.
   * Depending on the type of connection there may be a messsage header
   * indicating the length of the xml message body (http works this way).
   * @return amount of bytes to read, -1 means unknown length.
   */
   virtual long getBytesToRead() const = 0;

 /** Resets the state of the Protocol.
   * Before starting a transfer you should call this to ensure
   * a defined state of the internal state machine processing the
   * protocol.
   */
   virtual void resetConnection();

 /** Sends a MethodCall over the connection.
   * @param   call        pointer to the calling data
   * @param   resource    resource for rpc on remote host
   * @param   wbxml_mode  true: data is sent as wbxml
   */
   virtual void sendRpcCall(const MethodCall &call, const CppString &resource, bool wbxml_mode);

 /** Sends a MethodResponse over the connection.
   * @param   resp        pointer to the response data
   * @param   wbxml_mode  true: data is sent as wbxml
   */
   virtual void sendRpcResponse(const MethodResponse &resp, bool wbxml_mode);

 /** Sets the connections persistence.
   * Usually an xml rpc call closes after the response has been received.
   * To improve performance or to transfer other protocls you may set
   * the connection to remain open. It is closed at the latest when an
   * error occurs.
   * @param   pers  true if connection shall be persistent.
   */
   void setPersistent(bool pers)
     { persistent = pers; }

 /** Gets the persistence information.
   * @return  true if connection is persistent
   */
   bool isPersistent() const
     { return persistent; }

 /** Tests if the response was successful regarding the transportation.
   * @param   phrase  return value describing the problem.
   * @return true  resonse is OK.
   */
   virtual bool responseStatus(CppString &phrase) const = 0;

   typedef enum
   {
     ConnStart,                 // 0 start and reset
     ConnPendingCR,             // 1 just found CR, wait for LF or fold whitespace
     ConnPendingHeaderLine,     // 2 new line started, may be folded
     ConnHeaderLine,            // 3 collection chars for current header line
     ConnSwitchToBody,          // 4 header completely read ==> analyse header fields
     ConnBody,                  // 5 in body, do nothing. Never reached if there is no body!
     ConnError                  // 6 error occured, set status accordingly
   }
   State;


 /** Process a chunk of input data from the connection.
   * Usually you call read() to get some more data and pass it to
   * this method. It processes the header (depending on your protocol).
   * Normally you only have to take care of two states: ConnError which
   * results in terminating the connection. ConnBody on the other side
   * indicates user data which might be fed to a parser. @see Dispatcher.
   * @param  buffer   pointer to input data
   * @param len       valid length of buffer
   * @return   current state of the processing.
   */
   virtual State connectionMachine(char * &buffer, long &len);

  /** Adds another potential username, password and "range" for this connection (server mode).
   * @param  user   Username
   * @param  pass   Password
   * @param  realm  Synonym for area which shall be accessed
   */
   void addAuthentication(const CppString &user, const CppString &pass,
                          const CppString &realm);

 /** Sets username and password for the next transmission (client mode).
   * This makes the next transmission block use a simple authentication scheme
   * with a username and password.
   * @param  user   Username
   * @param  pass   Password
   */
   virtual void setMessageAuthentication(const CppString &user, const CppString &pass);

 /** Checks for a valid user in the current message.
   * There must be an appropriate AuthData set and the connection must support
   * username + password transmission (default is NOT to do so).
   * The caller is resonsible for sending a negative response if approriate.
   * @param  realm  Synonym for area which shall be accessed
   * @return true: access shall be granted.
   */
   virtual bool checkAuthentication(const CppString &realm) const;

 /** Sends a negative response to the caller
   * @param  realm   Description for the caller which username and password to pass in request
   * @return true: access shall be granted.
   */
   virtual void rejectAuthentication(const CppString &realm);

 /** Sets the current transmission to expect/accept no return value.
   */
   virtual void setTransmitOnly();

 /** Tests if the current transmission expects a return value.
   * @return true: return value for request
   */
   virtual bool isTransmitOnly();

 /** Returns the connection object.
   * @return pointer to connection object
   */
   Connection * getConnection();

 /** Detaches the protocol by creating a duplicate of
   * the protocol + connection and closing the original connection afterwards.
   * @return the current protocol
   */
   virtual Protocol *detach() = 0;
 
 protected:

 /** Extracts username and password from current message.
   * @param  user   reference to return username
   * @param  pass   reference to return password
   * @return true: username and password could be extracted
   */
   virtual bool getUserPass(CppString &user, CppString &pass) const;

 /** Determines the length of the message body.
   */
   virtual void determineContentLength() = 0;

  /** Returns the protocol name.
   * @return protocol name
   */
   virtual CppString getProtocolName() = 0;
  
 protected:
  
   Connection     *connection;
   bool            delete_connection;
   State           connstate;
   long            content_length;
   long            remain_content_length;

 private:

 /** Initializes internal variables.
   */
   void init();

   struct AuthData
   {
     AuthData(const CppString &user_, const CppString &pass_, const CppString &realm_)
      : user(user_)
      , pass(pass_)
      , realm(realm_)
     {}

     CppString  user;
     CppString  pass;
     CppString  realm;
   };

   bool      persistent;

   std::vector<AuthData>  authdata;
};


};  // namespace ulxr


#endif // ULXR_PROTOCOL_H

