/***************************************************************************
               ulxr_valueparse_base.h  -  parse rpc primitive values
                             -------------------
    begin                : Fri Jan 09 2004
    copyright            : (C) 2002-2004 by Ewald Arnold
    email                : ulxmlrpcpp@ewald-arnold.de

    $Id: ulxr_valueparse_base.h,v 1.3 2004/01/11 13:05:55 ewald-arnold Exp $

 ***************************************************************************/

/**************************************************************************
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ***************************************************************************/

#ifndef ULXR_VALUEPARSE_BASE_H
#define ULXR_VALUEPARSE_BASE_H

#include <ulxmlrpcpp.h>  // always first header

#include <ulxr_value.h>
#include <ulxr_xmlparse_base.h>


namespace ulxr {


/** Base class for WBXML RPC parsing.
  *
  * IMPORTANT:
  * The current "Value" is moved around via pointers and is not
  * automatically destroyed. The object taking over the "Value" resp. the object
  * storing the value somehow else must "delete" the "Value" it gets.
  *
  * @see ArrayState::takeValue
  */
class ULXR_EXPORT ValueParserBase
{
 public:

  /** Gets the final Value after parsing.
   * @return the Value.
   */
   Value getValue() const;

   enum State
   {
     eValue  = XmlParserBase::eXmlParserLast,
     eArray,   eData,
     eStruct,  eMember, eName,
     eBoolean, eInt,    eI4,   eDouble,
     eString,  eBase64, eDate,
     eValueParserLast
   };

/** Helper class to represent the data of the current parsing step.
  */
  class ULXR_EXPORT ValueState : public XmlParserBase::ParserState
  {
   public:

   /** Constructs an ValueState.
     * @param  the actual state
     */
     ValueState (unsigned st);

   /** Transfers a Value into the ValueState.
     * @param  val   the value
     */
     virtual void takeValue(Value *val);

   /** Transfers a member name into the ValueState.
     * Used only for Structs.
     * @param  name   the member name
     */
     virtual void takeName(const CppString & /* name */ )
       {}

   /** Gets the name of the state.
     * Useful only for debugging.
     * @return the name of actual state
     */
     virtual CppString getStateName() const;

   /** Gets the name of the member.
     * Used only with Structs
     * @return the member name
     */
     CppString getName() const
       { return name; }

   /** Sets the name of the member.
     * Used only with Structs
     * @param  name   the member name
     */
     void setName(const CppString &nm)
       { name = nm; }

   /** Gets the Value of this state.
     * @return the Value.
     */
     Value* getValue() const
       { return value; }

   protected:

     Value     *value;

   private:

     CppString  name;

     ValueState(const ValueState&); // forbid this
     ValueState& operator= (const ValueState&);
  };

 protected:

/** Helper class to represent the data of the current parsing step
  * when the xml element is a Struct.
  */
  class ULXR_EXPORT MemberState : public ValueState
  {
   public:

   /** Constructs a MemberState.
     * @param  st   the actual state
     * @param  str  pointer to the resulting value
     */
     MemberState(unsigned st, Value *val);

   /** Transfers a Value into the MemberState.
     * @param  val   the value
     */
     virtual void takeValue(Value *val);

   /** Transfers a member name into the MemberState.
     * @param  name   the member name
     */
     virtual void takeName(const CppString &n);

   protected:
     
     Value  *mem_val;

   private:
     MemberState(const MemberState&); // forbid this
     MemberState& operator= (const MemberState&);
  };


/** Helper class to represent the data of the current parsing step
  * when the xml element is an Array.
  */
  class ULXR_EXPORT ArrayState : public ValueState 
  {
   public:

   /** Constructs an ArrayState.
     * @param  st   the actual state
     */
     ArrayState(unsigned st);

   /** Transfers a Value into the ArrayState.
     * @param  val   the value
     */
     virtual void takeValue(Value *val);

   private:
     ArrayState(const ArrayState&); // forbid this
     ArrayState& operator= (const ArrayState&);
  };

   friend class ValueState;

 protected:

 /** Gets a pointer to the topmost ValueState.
   * @return pointer to ValueState
   */
   virtual ValueState *getTopValueState() const = 0;
};


};  // namespace ulxr


#endif // ULXR_VALUEPARSE_BASE_H
